"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Test = void 0;
const core_1 = require("@jest/core");
const core_2 = require("@oclif/core");
const commonFlags = require("../../common/flags");
const path = require("path");
const debug_1 = require("debug");
const config_1 = require("../../common/config");
const definition_1 = require("../../common/definition");
const debug = (0, debug_1.default)('cmd');
class Test extends core_2.Command {
    async run() {
        const { args, flags } = await this.parse(Test);
        const { dereference, validate, bundle, header } = flags;
        const definition = (0, definition_1.resolveDefinition)(args.definition);
        if (!definition) {
            this.error('Please load a definition file', { exit: 1 });
        }
        let document;
        try {
            document = await (0, definition_1.parseDefinition)({
                definition,
                dereference,
                bundle,
                validate,
                servers: flags.server,
                inject: flags.inject,
                strip: flags.strip,
                header,
                induceServers: true,
            });
        }
        catch (err) {
            this.error(err, { exit: 1 });
        }
        const testConfig = (0, config_1.getConfigValue)('tests');
        if (!testConfig) {
            this.error('Please run `test add` first', { exit: 1 });
        }
        const jestArgv = {
            ...flags,
            $0: 'jest',
            _: [],
            passWithNoTests: true,
            verbose: true,
        };
        // filter tests by operation
        if (flags.operation) {
            jestArgv.testNamePattern = flags.operation.map((o) => `${o} `).join('|');
        }
        const testProjectDir = path.dirname(require.resolve('../../tests/run-jest'));
        jestArgv.rootDir = testProjectDir;
        debug('jestArgv', jestArgv);
        await (0, core_1.runCLI)(jestArgv, [testProjectDir]);
    }
}
exports.Test = Test;
Test.description = 'Run automated tests against APIs';
Test.examples = [
    `$ openapi test`,
    `$ openapi test -o getPets`,
];
Test.flags = {
    ...commonFlags.help(),
    ...commonFlags.parseOpts(),
    operation: core_2.Flags.string({ char: 'o', description: 'filter by operationId', helpValue: 'operationId', multiple: true }),
    verbose: core_2.Flags.boolean({
        char: 'v',
        description: 'verbose mode',
        default: false,
    }),
    ...commonFlags.securityOpts(),
};
