"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = require("yargs");
const openapi_client_axios_1 = require("openapi-client-axios");
const config_1 = require("../common/config");
const definition_1 = require("../common/definition");
const security_1 = require("../common/security");
const debug_1 = require("debug");
const utils_1 = require("../common/utils");
const debug = (0, debug_1.default)('cmd');
const testConfig = (0, config_1.getConfigValue)('tests');
const argv = (0, yargs_1.default)(process.argv.slice(2))
    .option('server', { alias: 'S', type: 'string', array: true })
    .option('inject', { alias: 'I', type: 'string', array: true })
    .option('apiRoot', { alias: 'R', type: 'string' })
    .option('header', { alias: 'H', type: 'string', array: true })
    .option('security', { alias: 's', type: 'string', array: true })
    .option('apikey', { alias: 'k', type: 'string' })
    .option('token', { alias: 't', type: 'string' })
    .option('username', { alias: 'u', type: 'string' })
    .option('password', { alias: 'p', type: 'string' })
    .parseSync();
const definition = (0, definition_1.resolveDefinition)((_a = argv[0]) !== null && _a !== void 0 ? _a : (0, config_1.getConfigValue)('definition'));
let api;
beforeAll(async () => {
    const document = await (0, definition_1.parseDefinition)({
        definition,
        servers: argv.server,
        inject: argv.inject,
        header: argv.header,
        induceServers: true,
    });
    api = new openapi_client_axios_1.default({ definition: document });
});
for (const operationId of Object.keys(testConfig)) {
    describe(operationId, () => {
        for (const testName of Object.keys(testConfig[operationId])) {
            describe(testName, () => {
                const testDefinition = testConfig[operationId][testName];
                if (testDefinition.checks.includes('Success2XX')) {
                    test('should return 2XX response', async () => {
                        const client = await getClientForTest({ operationId, requestConfig: testDefinition.request.config });
                        const res = await client[operationId](testDefinition.request.params, testDefinition.request.data);
                        expect(res.status).toBeGreaterThanOrEqual(200);
                        expect(res.status).toBeLessThan(300);
                    });
                }
            });
        }
    });
}
const getClientForTest = async (params) => {
    const client = await api.init();
    const securityRequestConfig = await (0, security_1.createSecurityRequestConfig)({
        document: definition,
        operation: api.getOperation(params.operationId),
        security: argv.security,
        header: argv.header,
        apikey: argv.apikey,
        token: argv.token,
        username: argv.username,
        password: argv.password,
        noInteractive: true,
    });
    debug('securityRequestConfig %o', securityRequestConfig);
    // add cookies
    const cookies = {
        ...securityRequestConfig.cookie,
    };
    const cookieHeader = Object.keys(cookies)
        .map((key) => `${key}=${cookies[key]}`)
        .join('; ');
    // add request headers
    client.defaults.headers.common = {
        ...params.requestConfig.headers,
        ...securityRequestConfig.header,
        ...(0, utils_1.parseHeaderFlag)(argv.header),
        ...(Boolean(cookieHeader) && { cookie: cookieHeader }),
    };
    // add query params
    client.defaults.params = {
        ...params.requestConfig.params,
        ...securityRequestConfig.query,
    };
    // add basic auth
    client.defaults.auth = {
        ...params.requestConfig.auth,
        ...securityRequestConfig.auth,
    };
    return client;
};
